
/****************************************************************************
 *  Clips -- Schema SlotValue
 *  (auto-generated, do not modify directly)
 *
 *  CLIPS REST API.
 *  Enables access to CLIPS environments.
 *
 *  API Contact: Tim Niemueller <niemueller@kbsg.rwth-aachen.de>
 *  API Version: v1beta1
 *  API License: Apache 2.0
 ****************************************************************************/
/*  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version. A runtime exception applies to
 *  this software (see LICENSE.GPL_WRE file mentioned below for details).
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  Read the full text in the LICENSE.GPL_WRE file in the doc directory.
 */

#pragma once

#define RAPIDJSON_HAS_STDSTRING 1

#include <rapidjson/fwd.h>

#include <cstdint>
#include <memory>
#include <optional>
#include <string>
#include <vector>

/** SlotValue representation for JSON transfer. */
class SlotValue
{
public:
	/** Constructor. */
	SlotValue();
	/** Constructor from JSON.
	 * @param json JSON string to initialize from
	 */
	SlotValue(const std::string &json);
	/** Constructor from JSON.
	 * @param v RapidJSON value object to initialize from.
	 */
	SlotValue(const rapidjson::Value &v);

	/** Destructor. */
	virtual ~SlotValue();

	/** Get version of implemented API.
	 * @return string representation of version
	 */
	static std::string
	api_version()
	{
		return "v1beta1";
	}

	/** Render object to JSON.
	 * @param pretty true to enable pretty printing (readable spacing)
	 * @return JSON string
	 */
	virtual std::string to_json(bool pretty = false) const;
	/** Render object to JSON.
	 * @param d RapidJSON document to retrieve allocator from
	 * @param v RapidJSON value to add data to
	 */
	virtual void to_json_value(rapidjson::Document &d, rapidjson::Value &v) const;
	/** Retrieve data from JSON string.
	 * @param json JSON representation suitable for this object.
	 * Will allow partial assignment and not validate automaticaly.
	 * @see validate()
	 */
	virtual void from_json(const std::string &json);
	/** Retrieve data from JSON string.
	 * @param v RapidJSON value suitable for this object.
	 * Will allow partial assignment and not validate automaticaly.
	 * @see validate()
	 */
	virtual void from_json_value(const rapidjson::Value &v);

	/** Validate if all required fields have been set.
	 * @param subcall true if this is called from another class, e.g.,
	 * a sub-class or array holder. Will modify the kind of exception thrown.
	 * @exception std::vector<std::string> thrown if required information is
	 * missing and @p subcall is set to true. Contains a list of missing fields.
	 * @exception std::runtime_error informative message describing the missing
	 * fields
	 */
	virtual void validate(bool subcall = false) const;

	// Schema: SlotValue
public:
	/** Get name value.
   * @return name value
   */
	std::optional<std::string>
	name() const
	{
		return name_;
	}

	/** Set name value.
	 * @param name new value
	 */
	void
	set_name(const std::string &name)
	{
		name_ = name;
	}
	/** Get type value.
   * @return type value
   */
	std::optional<std::string>
	type() const
	{
		return type_;
	}

	/** Set type value.
	 * @param type new value
	 */
	void
	set_type(const std::string &type)
	{
		type_ = type;
	}
	/** Get is-multifield value.
   * @return is-multifield value
   */
	std::optional<bool>
	is_multifield() const
	{
		return is_multifield_;
	}

	/** Set is-multifield value.
	 * @param is_multifield new value
	 */
	void
	set_is_multifield(const bool &is_multifield)
	{
		is_multifield_ = is_multifield;
	}
	/** Get values value.
   * @return values value
   */
	std::vector<std::string>
	values() const
	{
		return values_;
	}

	/** Set values value.
	 * @param values new value
	 */
	void
	set_values(const std::vector<std::string> &values)
	{
		values_ = values;
	}
	/** Add element to values array.
	 * @param values new value
	 */
	void
	addto_values(const std::string &&values)
	{
		values_.push_back(std::move(values));
	}

	/** Add element to values array.
	 * The move-semantics version (std::move) should be preferred.
	 * @param values new value
	 */
	void
	addto_values(const std::string &values)
	{
		values_.push_back(values);
	}

private:
	std::optional<std::string> name_;
	std::optional<std::string> type_;
	std::optional<bool>        is_multifield_;
	std::vector<std::string>   values_;
};